public class QueueTest3 {
  
  /**
   * removeDuplicates: remove duplicates from a queue whose items are in non-decreasing order.
   * 
   * @param q   the given queue whose items are in non-decreasing order.
   * @return a queue with each item of the given queue appearing exactly once.
   */
  public static QueueReferenceBased removeDuplicates(QueueReferenceBased q)
  {
        QueueReferenceBased newQueue = new QueueReferenceBased();
        
        while (!q.isEmpty()) {
            // Get the next (new) item and add it to the new queue
            Comparable newItem = (Comparable)q.dequeue();
            newQueue.enqueue(newItem);
            
            // Skip duplicates
            while (!q.isEmpty() && newItem.compareTo((Comparable)q.peek()) == 0)
                q.dequeue();
        } // end while
        
        return newQueue;
  }

  /**
   * mergeQueues: merge the items of two queues (non-decreasing order) into a new queue.
   * 
   * @param queue1  the first queue
   * @param queue2  the second queue
   * @return the new queue
   * 
   * Preconditions: the items of the given queues are in non-decreasing order; no item appears 
   * more than once in any given queue; however the same item may appear in queue1 and queue2.
   * Postcondition: the items of the new queue are in non-decreasing order, and consist of
   * the items of the two given queues; duplicates do not appear in the new queue unless they
   * appear in one or both of the given queues.
   */
  public static QueueReferenceBased mergeQueues(QueueReferenceBased queue1, QueueReferenceBased queue2) {
    QueueReferenceBased queue3 = new QueueReferenceBased();
           
    // non-trivial case; both of them are not empty
    while (!queue1.isEmpty() && !queue2.isEmpty()) {
        int a = ((Integer)queue1.peek()).intValue();
        int b = ((Integer)queue2.peek()).intValue();
        if (a > b) {
            queue3.enqueue(queue2.dequeue());
        }
        else if (a < b) {
            queue3.enqueue(queue1.dequeue());
        }
        else {
            queue3.enqueue(queue1.dequeue());
            queue2.dequeue();
        }
    }
    
    // trivial case - queue1 is empty, queue2 is not empty
    if (queue1.isEmpty() && !queue2.isEmpty()) {
        while (!queue2.isEmpty()) {
            queue3.enqueue(queue2.dequeue());
        }
    }
    // trivial case - queue2 is empty, queue1 is not empty
    else if (queue2.isEmpty() && !queue1.isEmpty()) {
        while (!queue1.isEmpty()) {
            queue3.enqueue(queue1.dequeue());
        }
    }
    
    return queue3;
  }  // end mergeQueues
  
  public static void main(String[] args) {
    try {
      QueueReferenceBased queue1 = new QueueReferenceBased();  
      QueueReferenceBased queue2 = new QueueReferenceBased();
      QueueReferenceBased queue3 = new QueueReferenceBased();
      
      // Intialize the two given queues.
      queue1.enqueue(new Integer(1));
      queue1.enqueue(new Integer(1));
      queue1.enqueue(new Integer(1));
      queue1.enqueue(new Integer(1));
      queue1.enqueue(new Integer(2));
      queue1.enqueue(new Integer(3));
      queue1.enqueue(new Integer(5));    
    
      queue2.enqueue(new Integer(1));
      queue2.enqueue(new Integer(4));
      queue2.enqueue(new Integer(4));
      queue2.enqueue(new Integer(4));
      queue2.enqueue(new Integer(9));
      queue2.enqueue(new Integer(12));
    
      // Remove duplicates from the two given queues.
      queue1 = removeDuplicates(queue1);
      queue2 = removeDuplicates(queue2);      
    
      // Merge the two given queues into a new queue.
      queue3 = mergeQueues(queue1, queue2);
    
      // Print the new queue.
      while (!queue3.isEmpty()) {
        //int a = ((Integer)queue3.peek()).intValue();
        System.out.println((Integer)(queue3.dequeue()));
      }
   }
   catch(Exception e) {
        System.out.println("Exception in main");
        e.printStackTrace();    
   }
  } // end main
}  // QueueTest3